-- Changes by Darren New <dnew@san.rr.com>
--  Added "Test_Failed", added check of Encode_Length, Decode_Length.

with Base64,
     Ada.Streams,
     Ada.Streams.Stream_Io,
     Ada.Text_Io;
use Ada.Streams;
procedure Test64 is
  -- read N bytes from file "a", encode, decode, and see if original restored

  Test_Failed : Boolean := False;

  procedure Test(N : in Stream_Element_Offset) is
    Fd      : Stream_Io.File_Type;
    Output_Quad_Count: constant Integer := (Integer(N) + 2) / 3;
    Output_Byte_Count: constant Integer := 4 * Output_Quad_Count;
    Encoded : String(1 .. Output_Byte_Count + 2 * (Output_Byte_Count / 76));
    Last_E  : Integer;
    Buffer,
    Buffer2 : Stream_Element_Array(1 .. N);
    Last_D  : Stream_Element_Offset;
  begin
    Ada.Text_Io.Put_Line("test" & Stream_Element_Offset'Image(N));
    Stream_Io.Open(Fd, Stream_Io.In_File, "a");
    Stream_Io.Read(Fd, Buffer, Last_D);
    Base64.Encode(Buffer(1 .. Last_D), Encoded, Last_E);
    Ada.Text_Io.Put_Line("encoded in" & Integer'Image(Last_E));
    if Base64.Encode_Length(Last_D) /= Last_E then
        Ada.Text_Io.Put_Line("Encoded length does not match calculated length");
        Ada.Text_Io.Put_Line("Calculated = " & 
            Integer'Image(Base64.Encode_Length(Last_D)));
        Test_Failed := True;
    end if;
    Ada.Text_Io.Put(Encoded(1 .. Last_E));
    Ada.Text_Io.New_Line;
    Base64.Decode(Encoded(1 .. Last_E), Buffer2, Last_D);
    if Last_D /= Buffer2'last then
      Ada.Text_Io.Put_Line(Stream_Element_Offset'Image(Last_D)
        & "/="
        & Stream_Element_Offset'Image(Buffer2'last));
      Test_Failed := True;
    end if;
    if Buffer2(1 .. Last_D) /= Buffer then
      Ada.Text_Io.Put_Line("/=");
      Test_Failed := True;
    end if;
    Stream_Io.Close(Fd);
  end Test;

  procedure Test_Decode_Length is
    S1 : String := 
    "SGVsbG8gdGhlcmUgYWxsIHlvdSBjcmF6eSBwZW9wbGUhIEhvdyBhcmUgeW91IHRvZGF5Pw==";
    S2 : String := 
    "SGVsbG8gdGhlc***mUgYWxsI  H l   vdSBjcmF6eSBwZW9wbGUhIEhvdyBhcmUgeW91IHRvZGF5Pw==";
    S3 : String := 
    "SGVsbG8gdGhlcmUgYWxsIHlvdSBjcmF6-eSBwZW9wbGUhIEhvdyBhcmUgeW91IHRvZGF5Pw==";
    S4 : String := 
    "SGVsbG8gdGhlcmUgYWxsIHlvdSBjcmF6";
    S5 : String := 
    "SGVsbG8gdGhlcmUgYWxsIHlvdSBjcmF6eS";
    S1_SL:Natural := S1'Last - 2; S1_TL:Natural := 52; S1_C:Boolean := True;
    S2_SL:Natural := S2'Last - 2; S2_TL:Natural := 52; S2_C:Boolean := True;
    S3_SL:Natural := 32;          S3_TL:Natural := 24; S3_C:Boolean := True;
    S4_SL:Natural := S4'Last - 0; S4_TL:Natural := 24; S4_C:Boolean := False;
    S5_SL:Natural := S5'Last - 2; S5_TL:Natural := 24; S5_C:Boolean := False;
    SL : Natural; TL : Ada.Streams.Stream_Element_Offset; C : Boolean;
    TLI : Natural;
  begin
    -- Probably better done as arrays and such.
    Ada.Text_Io.Put_Line("Testing Decode_Length");
    Base64.Decode_Length(S1, SL, TL, C); TLI := Natural(TL);
    if S1_SL /= SL or S1_TL /= TLI or S1_C /= C then
        Ada.Text_Io.Put_Line("Decode_Length(S1) failed");
        Test_Failed := True;
    end if;
    Base64.Decode_Length(S2, SL, TL, C); TLI := Natural(TL);
    if S2_SL /= SL or S2_TL /= TLI or S2_C /= C then
        Ada.Text_Io.Put_Line("Decode_Length(S2) failed");
        Test_Failed := True;
    end if;
    Base64.Decode_Length(S3, SL, TL, C); TLI := Natural(TL);
    if S3_SL /= SL or S3_TL /= TLI or S3_C /= C then
        Ada.Text_Io.Put_Line("Decode_Length(S3) failed");
        Test_Failed := True;
    end if;
    Base64.Decode_Length(S4, SL, TL, C); TLI := Natural(TL);
    if S4_SL /= SL or S4_TL /= TLI or S4_C /= C then
        Ada.Text_Io.Put_Line("Decode_Length(S4) failed");
        Test_Failed := True;
    end if;
    Base64.Decode_Length(S5, SL, TL, C); TLI := Natural(TL);
    if S5_SL /= SL or S5_TL /= TLI or S5_C /= C then
        Ada.Text_Io.Put_Line("Decode_Length(S5) failed");
        Test_Failed := True;
    end if;
  end Test_Decode_Length;

  procedure Test_Encode_Decode_Stream is
    Text_Fd : Ada.Text_Io.File_Type;
    Data_Fd : Ada.Streams.Stream_Io.File_Type;
    Last_Line : String(1..500);
    Last_Len  : Natural;
  begin
    Ada.Text_Io.Create(Text_Fd, Mode => Ada.Text_Io.Out_File, Name => "b.txt");
    Ada.Streams.Stream_Io.Open(Data_Fd, Mode => Ada.Streams.Stream_Io.In_File, 
            Name => "a");
    Base64.Encode_Stream(From => Data_Fd, To => Text_Fd);
    Ada.Text_Io.Put_Line(Text_Fd, "--BoundaryMarker--");
    Ada.Text_Io.Close(Text_Fd);
    Ada.Streams.Stream_Io.Close(Data_Fd);
    Ada.Text_Io.Put_Line("b.txt created from a. Take a look, maybe edit it.");
    Ada.Text_Io.Put_Line("Then press return to change it back.");
    Ada.Text_Io.Flush;
    Ada.Text_Io.Skip_Line;
    Ada.Text_Io.Open(Text_Fd, Mode => Ada.Text_Io.In_File, Name => "b.txt");
    Ada.Streams.Stream_Io.Create(Data_Fd, 
            Mode => Ada.Streams.Stream_Io.Out_File, name => "c");
    Base64.Decode_Stream(From => Text_Fd, To => Data_Fd);
    Ada.Text_Io.Get_Line(Text_Fd, Last_Line, Last_Len);
    Ada.Text_Io.Put_Line("Output went to file 'c'. Last line of input is " &
        Last_Line(1..Last_Len));
    Ada.Text_Io.Close(Text_Fd);
    Ada.Streams.Stream_Io.Close(Data_Fd);
  end Test_Encode_Decode_Stream;

begin

  for I in Stream_Element_Offset range 9 .. 14 loop
    Test(I);
  end loop;

  for I in Stream_Element_Offset range 252 .. 290 loop
    Test(I);
  end loop;

  for I in Stream_Element_Offset range 1 .. 9 loop
    Test(I);
  end loop;

  Test_Decode_Length;

  if Test_Failed then
    Ada.Text_Io.Put_Line("One or more tests failed.");
  else
    Ada.Text_Io.Put_Line("All tests passed.");
  end if;

  Test_Encode_Decode_Stream;

end Test64;

